(* Structure containing types needed by the MWB XML
   parser, e.g. element names *)

structure MwbXmlParserTypes =
    struct
	(* Note: More document types can be added here later *)
	datatype DocumentType =
	    Definitions
	  | TransitionSystem
	  | Unknown

	fun getDocType (s) =
	    case s of "DEFINITIONS" => Definitions:DocumentType
	  | "TRANSITIONSYSTEM" => TransitionSystem:DocumentType
	  | _ => Unknown:DocumentType

	datatype TermType =
	    Nil
	  | Identifier
	  | Constructor
	  | Unknown

	fun getTermType (s) =
	    case s of "NIL" => Nil:TermType
	  | "IDENTIFIER" => Identifier:TermType
	  | "CONS" => Constructor:TermType
	  | "SUMMATION" => Constructor:TermType
	  | "PARALLEL" => Constructor:TermType
	  | "RESTRICTION" => Constructor:TermType
	  | "IF" => Constructor:TermType		
	  | _ => Unknown:TermType

	datatype ConstructorType =
	    Cons
	  | Summation
	  | Parallel
	  | Restriction
	  | If
	  | Unknown

	fun getConstructorType (s) =
	    case s of "CONS" => Cons:ConstructorType
	  | "SUMMATION" => Summation:ConstructorType
	  | "PARALLEL" => Parallel:ConstructorType
	  | "RESTRICTION" => Restriction:ConstructorType
	  | "IF" => If:ConstructorType
	  | _ => Unknown:ConstructorType

	datatype PrefixType =
	    In
	  | Out
	  | Tau
	  | Unknown

	fun getPrefixType (s) =
	    case s of "IN" => In:PrefixType
	  | "OUT" => Out:PrefixType
	  | "TAU" => Tau:PrefixType
	  | _ => Unknown:PrefixType

	datatype MatchType =
	    Equal
	  | NotEqual
	  | Unknown

	fun getMatchType (s) =
	    case s of "EQUAL" => Equal:MatchType
	  | "NOTEQUAL" => NotEqual:MatchType
	  | _ => Unknown:MatchType

    end;
