(* Contains utility functions, e.g. for printing *)

structure Printer =
    struct

	open HookData

	fun strToUpperHelper (charList, nil) =
	    charList
	  | strToUpperHelper (charList, first::rest) =
	    strToUpperHelper((Char.toUpper(first)::charList), rest)

	fun strToUpper (str) =
	    String.implode (rev(strToUpperHelper([], String.explode str)))

	fun printOpt (NONE, tabbing) =
	    (print(tabbing ^ "No attribute options present\n"))
	  | printOpt (SOME(data1, data2), tabbing) =
	    (print(tabbing ^ "Attr options: \"");
		   print(UniChar.Data2String data1);
		   print("\", \"");
		   print(UniChar.Data2String data2);
		   print("\"\n"));
			 
	fun printAtts (nil, aDtd, tabbing) =
			 (print(""))
	  | printAtts (((indx, AP_IMPLIED, opt)::rest), aDtd, tabbing) =
			 (print(tabbing ^ "Attr (implied): ");
			  print(UniChar.Data2String(Dtd.Index2AttNot(aDtd) indx) ^ "\n");
			  printAtts(rest, aDtd, tabbing))
	  | printAtts (((indx, AP_MISSING, opt)::rest), aDtd, tabbing) =
			 (print(tabbing ^ "Attr (missing): ");
			  print(UniChar.Data2String(Dtd.Index2AttNot(aDtd) indx) ^ "\n");
			  printAtts(rest, aDtd, tabbing))
	  | printAtts (((indx, AP_DEFAULT(attr, attrval, _), opt)::rest), aDtd, tabbing) =
			 (print(tabbing ^ "Attr (default): ");
			  print (UniChar.Data2String(Dtd.Index2AttNot(aDtd) indx) ^ " = \"");
			  print (UniChar.Vector2String attrval ^ "\"\n");
			  printAtts(rest, aDtd, tabbing))
	  | printAtts (((indx, AP_PRESENT(attr, attrval, _), opt)::rest), aDtd, tabbing) =
			 (print(tabbing ^ "Attr (present): ");
			  print (UniChar.Data2String(Dtd.Index2AttNot(aDtd) indx) ^ " = \"");
			  print (UniChar.Vector2String attrval ^ "\"\n");
			  printAtts(rest, aDtd, tabbing));

    end