structure FatTree = 
    struct
	(* Data type returned by the XML document parser *)
	open HookData
	datatype ParseTree =
	    XmlDoc of XmlInfo * ParseTree list * Dtd.Dtd
	  |  ProcInst of ProcInstInfo
	  |  Comment of CommentInfo
	  |  White of WhiteInfo
	  |  Decl of DeclInfo
	  |  Element of StartTagInfo * ParseTree list * EndTagInfo
	  |  CData of CDataInfo
	  |  Data of DataInfo
	  |  CharRef of CharRefInfo
	  |  GenRef of GenRefInfo
	  |  ParRef of ParRefInfo
	  |  EntEnd of EntEndInfo
	  |  DocType of DtdInfo
	  |  Subset of SubsetInfo
	  |  ExtSubset of ExtSubsetInfo
	  |  EndDtd of EndDtdInfo;
	exception InternalError of string
    (*    val sundry=XlTools.sundry *)
    end;


structure XmlParseHooks:Hooks =
    struct
	open Dtd Errors NullOptions FatTree MwbXmlParserTypes

	type AppData  = bool * ParseTree list * Dtd
	type AppFinal = bool * ParseTree list * Dtd
	val dummyEndTagInfo = ((ErrorData.nullPosition,ErrorData.nullPosition),0,NONE)

	fun addToStack [XmlDoc (docinfo,l,dtd)] i = [XmlDoc (docinfo,i::l,dtd)]
	  |   addToStack (Element (sti,ptl,eti) :: stacktail) i = Element (sti,i::ptl,eti) :: stacktail
	  |   addToStack ptl i = raise InternalError "XmlParseHooks.addToStack"

	fun hookXml ((status,_,dtd),docinfo) = (status,[XmlDoc (docinfo,[],dtd)],dtd)

	fun hookFinish (status,[XmlDoc (docinfo,ptl,dtdo)],dtd) = (status, [XmlDoc (docinfo,rev ptl,dtd)], dtd)
	  |   hookFinish spd = spd

	fun hookProcInst ((status,ptl,dtd),pi) = (status,addToStack ptl (ProcInst pi),dtd)

	fun hookComment (a, ci) = a

	fun hookWhite (a, wi) = a

	fun hookDecl ((status,ptl,dtd),decl) = (status,addToStack ptl (Decl decl),dtd) 

	fun hookStartTag ((status,ptl,dtd),sti as (se,i,atts,n,empty))
	    = if empty
		  then (status,addToStack ptl (Element (sti,[],dummyEndTagInfo)),dtd)
	      else (status,(Element (sti,[],dummyEndTagInfo))::ptl,dtd)

	fun hookEndTag ((status,(Element (sti,ptl,eti))::stacktail,dtd),neweti)
	    = (status,addToStack stacktail (Element (sti,rev ptl,neweti)),dtd)
	  |   hookEndTag ((status,_,dtd),neweti) = raise InternalError "XmlParseHooks.hookEndTag"

	fun hookData (a, di) = a
	(* fun hookData ((status,ptl,dtd),di) = (status,addToStack ptl (Data di),dtd) *)

	fun hookCData ((status,ptl,dtd),cdi) = (status,addToStack ptl (CData cdi),dtd)

	fun hookCharRef ((status,ptl,dtd),cri) = (status,addToStack ptl (CharRef cri),dtd)

	fun hookGenRef ((status,ptl,dtd),gri) = (status,addToStack ptl (GenRef gri),dtd)

	fun hookParRef ((status,ptl,dtd),pri) = (status,addToStack ptl (ParRef pri),dtd)

	fun hookEntEnd ((status,ptl,dtd),eei) = (status,addToStack ptl (EntEnd eei),dtd)
	fun hookDocType (a, dti) = a
	(* fun hookDocType ((status,ptl,dtd),dti) = (status,addToStack ptl (DocType dti),dtd) *)

	fun hookSubset (a, si) = a
	(* fun hookSubset ((status,ptl,dtd),si) = (status,addToStack ptl (Subset si),dtd) *)

	fun hookExtSubset (a, esi) = a
	(* fun hookExtSubset ((status,ptl,dtd),esi) = (status,addToStack ptl (ExtSubset esi),dtd) *)

	fun hookEndDtd (a, edi) = a
	(* fun hookEndDtd ((status,ptl,dtd),edi) = (status,addToStack ptl (EndDtd edi),dtd) *)
	    
	fun printError(pos,err) = if !O_SILENT then () else TextIO.output
	    (!O_ERROR_DEVICE,formatMessage (4,!O_ERROR_LINEWIDTH) 
	     (Position2String pos^" Error:"::errorMessage err))
	    
	fun printWarning(pos,warn) = if !O_SILENT then () else TextIO.output
	    (!O_ERROR_DEVICE,formatMessage (4,!O_ERROR_LINEWIDTH) 
	     (Position2String pos^" Warning:"::warningMessage warn))
	    
	fun hookError ((status,ptl,dtd),pe) = (false,ptl,dtd) before printError pe
	    
	fun hookWarning ((status,ptl,dtd),pw) = (status,ptl,dtd) before printWarning pw
    end;


